import falcon
from falcon.testing import TestCase, SimpleTestResource

import decimal
import datetime
from unittest import mock

from falcon_hateoas.middleware import JsonMiddleware


class TestJsonMiddleware(TestCase):

    def setUp(self):
        self.api = falcon.API(middleware=JsonMiddleware())

    def test_basic_data(self):
        self.api.add_route('/', SimpleTestResource(body={'a': 1}))
        resp = self.simulate_get()
        assert resp.json == {'a': 1}

    @mock.patch('sqlalchemy.ext.declarative.DeclarativeMeta')
    def test_alchemy_model(self, alchemy_mock):
        testdate = datetime.datetime.today()
        testinterval = datetime.timedelta(1, 2, 30, 40, 5, 6, 1)
        alchemy_mock.__table__ = mock.Mock()
        alchemy_mock.__table__.columns = {
            'testdate': testdate,
            'testinterval': testinterval,
            'testdecimal': decimal.Decimal('10.1'),
            'teststr': 'teststring'
        }
        alchemy_mock.__dict__.update(alchemy_mock.__table__.columns)

        self.api.add_route('/', SimpleTestResource(body=alchemy_mock))
        resp = self.simulate_get()
        assert resp.json == {
                'testdate': testdate.isoformat(),
                'testinterval': '8 days, 6:05:02.040030',
                'testdecimal': 10.1,
                'teststr': 'teststring'
        }



import falcon
import sqlalchemy


class ModelCollection:

    def on_get(self, req, resp):
        params = {
            'limit': None,
            'offset': 0,
            'order_by': None
        }
        params.update(req.params)
        result = self.session.query(self.model_class)
        filter = self._prepare_filter(params)
        if len(filter) > 0:
            result = result.filter(*filter)
        if params['order_by'] is not None:
            result = result.order_by(params['order_by'])
        if params['limit'] is not None:
            result = result.limit(params['limit'])
        result = result.offset(params['offset'])
        resp.body = result.all()

    def _prepare_filter(self, params):
        filter = []
        for k, v in params.items():
            column = getattr(self.model_class.__table__.columns, k, None)
            if column is not None:
                filter.append(column == v)
            else:
                split = k.rsplit('__', 1)
                if len(split) != 2:
                    continue
                else:
                    col = split[0]
                    op = split[1]
                column = getattr(self.model_class.__table__.columns, col, None)
                if column is None:
                    continue
                if isinstance(v, str):
                    if op == 'startswith':
                        filter.append(column.startswith(v))
                    elif op == 'endswith':
                        filter.append(column.endswith(v))
                    elif op == 'contains':
                        filter.append(column.contains(v))
                if op == 'lt':
                    filter.append(column < v)
                elif op == 'gt':
                    filter.append(column > v)
                elif op == 'lte':
                    filter.append(column <= v)
                elif op == 'gte':
                    filter.append(column >= v)
        return filter


class ModelResource:

    def on_get(self, req, resp, **kwargs):
        result = self.session.query(self.model_class)
        result = result.filter_by(**kwargs)
        try:
            resp.body = result.one()
        except sqlalchemy.orm.exc.NoResultFound:
            raise falcon.HTTPNotFound



import json
import decimal
import datetime
import sqlalchemy


class AlchemyJSONEncoder(json.JSONEncoder):
    def _is_alchemy_object(self, obj):
        try:
            sqlalchemy.orm.base.object_mapper(obj)
            return True
        except sqlalchemy.orm.exc.UnmappedInstanceError:
            return False

    def default(self, o):
        if self._is_alchemy_object(o):
            d = {}
            for col in o.__table__.columns.keys():
                value = getattr(o, col)
                if hasattr(value, 'isoformat'):
                    d[col] = value.isoformat()
                elif isinstance(value, datetime.timedelta):
                    d[col] = str(value)
                elif isinstance(value, decimal.Decimal):
                    d[col] = float(value)
                else:
                    d[col] = value
            return d
        else:
            return super(AlchemyJSONEncoder, self).default(o)


class JsonMiddleware:
    def process_response(self, req, resp, resource):
        resp.set_header('Content-Type', 'application/json; charset=utf-8')
        resp.body = json.dumps(resp.body, cls=AlchemyJSONEncoder)
